package com.keystar.plane.inspection.service.impl;

import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.keystar.plane.inspection.bo.PhotoBo;
import com.keystar.plane.inspection.bo.PlaneInspectionPicBo;
import com.keystar.plane.inspection.bo.PlaneInspectionResultBo;
import com.keystar.plane.inspection.bo.PlaneInspectionResultDetailBo;
import com.keystar.plane.inspection.entity.PhotoResultEntity;
import com.keystar.plane.inspection.entity.PlanePhotoEntity;
import com.keystar.plane.inspection.mapper.PhotoResultMapper;
import com.keystar.plane.inspection.mapper.PlanePhotoMapper;
import com.keystar.plane.inspection.service.PhotoResultService;
import com.keystar.plane.inspection.service.PlanePhotoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

@Service
public class PhotoResultServiceImpl extends ServiceImpl<PhotoResultMapper, PhotoResultEntity> implements PhotoResultService {

    @Resource
    PhotoResultMapper photoResultMapper;

    @Resource
    PlanePhotoMapper planePhotoMapper;

    @Autowired
    private MinioService minioService;

    @Autowired
    private PlanePhotoService planePhotoService;

    @Override
    public PhotoBo queryResultByPhotoId(PhotoBo photoBo) {
        Long id = photoBo.getId();
        List<PhotoResultEntity> photoResultEntities = photoResultMapper.queryResultByPhotoId(id);
        photoBo.setResults(photoResultEntities);
        return photoBo;
    }

    @Override
    public String pic2Base64FromMinio(String minioAddr) {

        String[] paths = StrUtil.splitToArray(minioAddr, "/");

        InputStream inputStream = null;
        ByteArrayOutputStream byteArrayOutputStream = null;

        try {
            inputStream = minioService.getObject(paths[1], StrUtil.sub(minioAddr,
                    CharSequenceUtil.length(paths[1]) + 2, CharSequenceUtil.length(minioAddr)));

            byteArrayOutputStream = new ByteArrayOutputStream();

            byte[] bytes = new byte[10240];
            int mReadLength = 0;

            while ((mReadLength = inputStream.read(bytes)) != -1) {
                byteArrayOutputStream.write(bytes, 0, mReadLength);
            }

            return Base64.getEncoder().encodeToString(byteArrayOutputStream.toByteArray());
        }catch (Exception e) {

        }finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                }catch (Exception e) {

                }
            }
            if (byteArrayOutputStream != null) {
                try {
                    byteArrayOutputStream.close();
                }catch (Exception e) {

                }
            }
        }

        return "";
    }

    @Override
    public String pic2Base64FromPath(String filePath) {

        FileInputStream fileInputStream = null;
        ByteArrayOutputStream byteArrayOutputStream = null;

        try {

            fileInputStream = new FileInputStream(filePath);

            byteArrayOutputStream = new ByteArrayOutputStream();

            byte[] bytes = new byte[10240];
            int mReadLength = 0;

            while ((mReadLength = fileInputStream.read(bytes)) != -1) {
                byteArrayOutputStream.write(bytes, 0, mReadLength);
            }

            return Base64.getEncoder().encodeToString(byteArrayOutputStream.toByteArray());
        }catch (Exception e) {

        }finally {
            if (fileInputStream != null) {
                try {
                    fileInputStream.close();
                }catch (Exception e) {

                }
            }
            if (byteArrayOutputStream != null) {
                try {
                    byteArrayOutputStream.close();
                }catch (Exception e) {

                }
            }
        }

        return "";

    }

    @Override
    public List<PlaneInspectionResultBo> listInspectionResult() {
        //查库组装巡检结果集合传入函数即可
        List<PlaneInspectionResultBo> planeInspectionResultBoList = new ArrayList<>();

        //查询有效航线中的最新架次
        List<PlanePhotoEntity> photoEntityList = planePhotoService.queryNewRecordByMission();
        log.debug("*******************photoEntityList Size: " + photoEntityList.size());
        for (PlanePhotoEntity photoEntity : photoEntityList){

            // 根据架次获取图片和异常信息
            List<PlanePhotoEntity> photoList = planePhotoMapper.queryPhotoByRecordId(photoEntity.getRecordId());
            log.debug("*******************photoList Size: " + photoList.size());
            for (PlanePhotoEntity photo : photoList){
                int detailNum = 0;
                PlaneInspectionResultBo planeInspectionResultBo = new PlaneInspectionResultBo();
                planeInspectionResultBo.setPics(new ArrayList<>());
                planeInspectionResultBo.setDetails(new ArrayList<>());

                int detailDioe = 0, detailHtspte = 0, detailOce = 0;
                planeInspectionResultBo.setId(String.valueOf(photo.getId()));
                PlaneInspectionPicBo planeInspectionPicBo = new PlaneInspectionPicBo();
//                planeInspectionPicBo.setVisible(pic2Base64FromPath("F:\\photo\\" + photo.getVisibleImagePath()));
                planeInspectionPicBo.setInfrared(pic2Base64FromPath("F:\\photo\\" +photo.getInfraredImagePath()));
                planeInspectionPicBo.setVisible("");
                planeInspectionResultBo.getPics().add(planeInspectionPicBo);

                // 根据图片id获取识别异常信息
                List<PhotoResultEntity> photoResultEntities = photoResultMapper.queryResultByPhotoId(photo.getId());
                if (photoResultEntities.size() != 0){
                    for (PhotoResultEntity photoResultEntity : photoResultEntities){
                        switch (photoResultEntity.getType()){
                            case "diodeFault":
                                detailDioe = 1;
                                break;
                            case "covered":
                                detailHtspte = 1;
                            default:

                        }
                    }
                }
                PlaneInspectionResultDetailBo planeInspectionResultDetailBo = new PlaneInspectionResultDetailBo();
                planeInspectionResultDetailBo.setNum(String.valueOf(detailNum+1));
                planeInspectionResultDetailBo.setName(photo.getPvName());
                planeInspectionResultDetailBo.setDioe(String.valueOf(detailDioe));
                planeInspectionResultDetailBo.setHtspte(String.valueOf(detailHtspte));
                planeInspectionResultDetailBo.setOce(String.valueOf(detailOce));
                planeInspectionResultBo.getDetails().add(planeInspectionResultDetailBo);

                planeInspectionResultBoList.add(planeInspectionResultBo);
            }
        }

        return planeInspectionResultBoList;
    }
}
