package com.keystar.plane.inspection.service.impl;

import cn.hutool.core.util.ArrayUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.keystar.plane.inspection.bo.DateBo;
import com.keystar.plane.inspection.bo.PhotoBo;
import com.keystar.plane.inspection.bo.PvNameBo;
import com.keystar.plane.inspection.constant.MissionConstant;
import com.keystar.plane.inspection.entity.PlanePhotoEntity;
import com.keystar.plane.inspection.mapper.PlanePhotoMapper;
import com.keystar.plane.inspection.service.PlanePhotoService;
import com.keystar.plane.inspection.utils.Utils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Service
@RequiredArgsConstructor
public class PlanePhotoServiceImpl extends ServiceImpl<PlanePhotoMapper, PlanePhotoEntity> implements PlanePhotoService {

    @Resource
    PlanePhotoMapper planePhotoMapper;

    @Override
    public List<PhotoBo> queryPhotoByPvName(PvNameBo pvNameBo) {
        Integer startIndex = pvNameBo.getStartIndex();
        pvNameBo.setStartIndex((startIndex-1)*pvNameBo.getPageSize());
        List<PlanePhotoEntity> photoEntityList = planePhotoMapper.getPhotoBo(pvNameBo);
        List<PhotoBo> photoBoList = new ArrayList<>();
        for( PlanePhotoEntity photoEntity : photoEntityList){
            PhotoBo photoBo = new PhotoBo();
            photoBo.setId(photoEntity.getId());
            photoBo.setMissionId(photoEntity.getMissionId());
            photoBo.setRecordId(photoEntity.getRecordId());
            photoBo.setVisibleImagePath(photoEntity.getVisibleImagePath());
            photoBo.setInfraredImagePath(photoEntity.getInfraredImagePath());
            photoBo.setTime(photoEntity.getTime());
            photoBo.setPzName(photoEntity.getPzName());
            // 将组串名字分割
            String pvNames = photoEntity.getPvName();
            if (pvNames != null){
                String[]  split = pvNames.split(",");
                List<String> pvNameList = new ArrayList<>(Arrays.asList(split));
                photoBo.setPvName(pvNameList);
            }
            // 将组串坐标分割
            String arrayBox = photoEntity.getArrayBox();
            if (arrayBox != null){
                String[] arrBoxSplit = arrayBox.split(";");
                List<String> arrBoxList = new ArrayList<>(Arrays.asList(arrBoxSplit));
                photoBo.setArrayBox(arrBoxList);
            }

            photoBoList.add(photoBo);
        }
        return photoBoList;
    }

    @Override
    public Integer getTotalSortie() throws ParseException {
        DateBo dateBo = Utils.getMonthDay();
        List<PlanePhotoEntity> recordIdNum = planePhotoMapper.getRecordIdNum(dateBo);
        return recordIdNum.size();
    }

    @Override
    public Integer getEffectiveSortie() throws ParseException {
        DateBo dateBo = Utils.getMonthDay();
        List<PlanePhotoEntity> photoEntityList = planePhotoMapper.getRecordIdNum(dateBo);
        int effectiveSortie = 0;
        if (photoEntityList.size() != 0){
            // 只有在固定航线上的架次才是有效架次
            for (PlanePhotoEntity photoEntity : photoEntityList){
                if (ArrayUtil.contains(MissionConstant.MISSION_ID, photoEntity.getMissionId())){
                    effectiveSortie+=1;
                }
            }
        }
        return effectiveSortie;
    }

    @Override
    public Integer getEffectiveInspectionDate() throws ParseException {
        DateBo dateBo = Utils.getMonthDay();
        List<PlanePhotoEntity> inspectionDate = planePhotoMapper.getInspectionDate(dateBo);
        return inspectionDate.size();
    }

    @Override
    public Integer getFltFZ() throws ParseException {
        DateBo dateBo = Utils.getMonthDay();
        List<PlanePhotoEntity> abnormalFZNum = planePhotoMapper.getAbnormalFZNum(dateBo);
        return abnormalFZNum.size();
    }

    @Override
    public List<PlanePhotoEntity> queryNewRecordByMission() {

        List<PlanePhotoEntity> photoList = new ArrayList<>();
        for (int i = 0; i < 3; i++) {
            Integer mission = MissionConstant.MISSION_ID[i];
            List<PlanePhotoEntity> photoEntityList = planePhotoMapper.queryNewRecordByMission(mission);
            PlanePhotoEntity photoEntity = photoEntityList.get(0);
            photoList.add(photoEntity);
        }
        return photoList;
    }

}