package com.keystar.plane.inspection.service.impl;

import com.keystar.plane.inspection.constant.MissionConstant;
import com.keystar.plane.inspection.entity.PhotoResultEntity;
import com.keystar.plane.inspection.entity.PlanePhotoEntity;
import com.keystar.plane.inspection.mapper.PhotoResultMapper;
import com.keystar.plane.inspection.service.PlanePhotoService;
import com.keystar.plane.inspection.service.TangyangDataService;
import com.keystar.plane.inspection.utils.FileUtil;
import com.keystar.plane.inspection.utils.HttpUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class TangyangDataServiceImpl implements TangyangDataService {

    @Resource
    PhotoResultMapper photoResultMapper;

    @Autowired
    PlanePhotoService planePhotoService;

    @Override
    public Double getMonthEquHour() {
        // 获取当前月份
        SimpleDateFormat monthFormat = new SimpleDateFormat("yyyy-MM-dd");
        String format = monthFormat.format(new Date());
        String url = "http://10.138.3.82:8080/home/getMonthEquHour/" + format;
        log.info("请求url地址:" + url);
        String s = HttpUtils.doGet(url, null);
        log.info("返回结果:" + s);
        return Double.parseDouble(s);
    }

    /**
     *  计算当月热斑损失电量
     **/
    @Override
    public Double getHtsptLose() {
        // 额定功率P = 535MWp
        Double p = 535.0;
        // 月利用小时数
//        Double monthEquHour = this.getMonthEquHour();
        Double monthEquHour = 35.15;
        // 获取每个航线的最新架次，计算全部的热斑组件数量
        Double coveredNum = 0.0;
//        for (int i = 0; i < 3; i++) {
//            String recordId = FileUtil.getRecordIdByMissionId(MissionConstant.MISSION_ID[i]);
//
//            List<PhotoResultEntity> covered = photoResultMapper.queryFltPVPanel(Integer.valueOf(recordId), "covered");
//            coveredNum = coveredNum + covered.size();
//        }
        List<PlanePhotoEntity> photoEntityList = planePhotoService.queryNewRecordByMission();
        for (PlanePhotoEntity photoEntity : photoEntityList){
            Integer recordId = photoEntity.getRecordId();
            List<PhotoResultEntity> covered = photoResultMapper.queryFltPVPanel(recordId, "covered");
            coveredNum = coveredNum + covered.size();
        }
        // 热斑的损失电量
        return p * coveredNum * monthEquHour * 0.32;

    }

    @Override
    public Double getDioLose() {
        // 额定功率P = 535MWp
        Double p = 535.0;
        // 月利用小时数
//        Double monthEquHour = this.getMonthEquHour();
        Double monthEquHour = 35.15;
        // 获取每个航线的最新架次，计算全部的热斑组件数量
        Double coveredNum = 0.0;
//        for (int i = 0; i < 3; i++) {
//            String recordId = FileUtil.getRecordIdByMissionId(MissionConstant.MISSION_ID[i]);
//
//            List<PhotoResultEntity> covered = photoResultMapper.queryFltPVPanel(Integer.valueOf(recordId), "diodeFault");
//            coveredNum = coveredNum + covered.size();
//        }
        List<PlanePhotoEntity> photoEntityList = planePhotoService.queryNewRecordByMission();
        for (PlanePhotoEntity photoEntity : photoEntityList){
            Integer recordId = photoEntity.getRecordId();
            List<PhotoResultEntity> covered = photoResultMapper.queryFltPVPanel(recordId, "diodeFault");
            coveredNum = coveredNum + covered.size();
        }
        // 二极管故障的损失电量
        return p * coveredNum * monthEquHour * 0.33;
    }
}
